﻿# ###########################################################
#
# Name : graphique.py
# Rôle : Bibliothèque permettant une utilisation
#        simplifiée de la bibliothèque pygame.
#
# Author:      Jean-Christophe Dagnet - Lycée Louis Bascan
# Sources :    Inspiré de la bibliothèque graphics.c - UVSQ site de Versailles
#
# Created:     04/04/2019
# Version:     2.0
# MAJ :        04/2023
# Copyright:   (c) JC 2019
#
# ###########################################################
#
#      Lycee Louis Bascan -- Rambouillet
#      Specialite N.S.I
#
# Professeur : Jean-Christophe DAGNET
#
# ############################################################
#
#   SOMMAIRE :
#
#   1. GESTION D'EVENNEMENTS            Ligne 59
#      - Fermeture de fenêtre           Ligne 64
#      - Clic souris (bloquant)         Ligne 85
#      - Gestion souris (non bloquant)  Ligne 130
#      - Gestion clavier                Ligne 180
#   2. AFFICHAGE DE TEXTE               Ligne 247
#   3. GESTION D'IMAGES                 Ligne 271
#   4. GESTION DES SONS ET MUSIQUES     Ligne 324
#      - PARTIE 4.1 : SONS              Ligne 329
#      - PARTIE 4.2 : Musiques          Ligne 366
#   5 GESTION DU TEMPS                  Ligne 443
#   6. VARIABLES COULEURS               Ligne 475
#           - Fançais                   Ligne 480
#           - Anglais                   Ligne 502
#
# ############################################################


# Importation de la bibliothèque Pygame et de ses constantes
import pygame
from pygame.locals import *

# Initialisation de tous les modules de la bibliothèque pygame
pygame.init()

# Changement de l'icone et du titre
icon_NSI = pygame.image.load("NSI.png")
pygame.display.set_icon(icon_NSI)
pygame.display.set_caption("Lycée Louis Bascan Rambouillet                          Formation N.S.I.")


#----------------------------------------------------------#
#-----------------                         ----------------#
#                 1. GESTION D'EVENNEMENTS                 #
#-----------------                         ----------------#
#----------------------------------------------------------#

###########################################################
# PARTIE 1 : Fonction bloquante d'attente de fermeture de la fenêtre
# suite à un évenement QUIT (ctrl+F4 ou clic sur la croix rouge)
###########################################################

def wait_escape():
    """
    Fonction blocante. Attente de fermeture de la fenêtre SDL
    """
    continuer = 1
    while continuer:
        pygame.time.wait(200)
        for event in pygame.event.get():  # On parcours la liste de tous les événements reçus
            if event.type == QUIT:        # Si un de ces événements est de type QUIT
                continuer = 0             # On arrête la boucle
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                continuer = 0
    pygame.quit()                         # on ferme la fenêtre
    return


###########################################################
# PARTIE 2 : Fonctions bloquantes d'attente d'un clic de souris
###########################################################

def wait_clic():
    """
    Fonction blaquante. Attente d'un clic de souris.
    Sortie : tuple des coordonnées du point cliqué
    """
    ev = pygame.event.wait()
    while(ev.type != MOUSEBUTTONDOWN):
        ev = pygame.event.wait()
    return ev.pos

def wait_left_clic():
    """
    Fonction blaquante. Attente d'un clic gauche de souris.
    Sortie : tuple des coordonnées du point cliqué
    """
    while 1:
        ev = pygame.event.wait()
        if (ev.type == MOUSEBUTTONDOWN and ev.button == 1):
            return ev.pos

def wait_center_clic():
    """
    Fonction blaquante. Attente d'un clic central de souris.
    Sortie : tuple des coordonnées du point cliqué
    """
    while 1:
        ev = pygame.event.wait()
        if (ev.type == MOUSEBUTTONDOWN and ev.button == 2):
            return ev.pos

def wait_right_clic():
    """
    Fonction blaquante. Attente d'un clic droit de souris.
    Sortie : tuple des coordonnées du point cliqué
    """
    while 1:
        ev = pygame.event.wait()
        if (ev.type == MOUSEBUTTONDOWN and ev.button == 3):
            return ev.pos


###########################################################
# PARTIE 3 : Fonctions non bloquantes de la souris
###########################################################

def device_mouse_off() :
    """
    Supprime le curseur de la souris dans la fenêtre graphique
    """
    pygame.mouse.set_visible(0)


def device_mouse_on() :
    """
    Affiche le curseur de la souris dans la fenêtre graphique
    """
    pygame.mouse.set_visible(1)


def get_mouse() :
    """
    Renvoie la position de la souris
    Instruction non bloquante
    """
    mouse = pygame.mouse.get_pos()
    return Point(mouse[0],mouse[1])


def is_mouse_pressed_left() :
    """
    Renvoie True si le bouton gauche de la souris est cliqué
    Instruction non bloquante
    """
    pygame.event.get()
    return pygame.mouse.get_pressed()[0]


def is_mouse_pressed_right() :
    """
    Renvoie True si le bouton droit de la souris est cliqué
    Instruction non bloquante
    """
    pygame.event.get()
    return pygame.mouse.get_pressed()[2]

def is_mouse_focused():
    """
    Renvoie True si la fenêtre graphique est sélectionnée
    """
    return pygame.mouse.get_focused()

###########################################################
# PARTIE 4 : Gestion du clavier
###########################################################

def wait_key():
    """
    Attend que l'on tape sur une touche du clavier (y compris
    une combinaison de touches) et renvoie le caractère correspondant.
    Instruction bloquante
    """
    if PYGAME_SDL_AFFICHAGE == 1:
        affiche_all()

    pygame.event.clear()

    caractere = ""
    while caractere == "":
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                caractere = event.dict['unicode']
                if caractere != "":
                    return caractere


def wait_space_letter():
    """
    Attend que l'on presse une touche du clavier.
    Si la touche est une lettre, renvoie la lettre correspondante en majuscule.
    Si la touche est la barre d'espace, renvoie la chaîne de caractère espace.
    Sinon, renvoie une chaîne vide.
    Instruction bloquante.
    """
    key = wait_key()
    if 96 < key < 123:
        return chr(key - 32)
    if key == 32:
        return " "
    return ""


def wait_arrow():
    """
    Attend que l'on presse une touche du clavier.
    Renvoie "up", "down", "left" ou  "right" suivant que l'on a
    tapé sur la flèche du haut, du bas, de gauche ou de droite.
    Renvoie une chaîne vide sinon.
    Instruction bloquante.
    """
    if PYGAME_SDL_AFFICHAGE == 1:
        affiche_all()

    pygame.event.clear()

    while 1:
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                if event.key == K_UP:
                    return "up"
                elif event.key == K_DOWN:
                    return "down"
                elif event.key == K_LEFT:
                    return "left"
                elif event.key == K_RIGHT:
                    return "right"


#----------------------------------------------------------#
#-----------------                         ----------------#
#                   2. AFFICHAGE DE TEXTE                  #
#-----------------                         ----------------#
#----------------------------------------------------------#

def aff_pol(fenetre, texte, taille, p_x, p_y, coul, text_bold=False, text_italic=False):
    """
    Affiche une chaîne de caractère en police Verdana
    Paramètres d'entrées :
        - fenetre : fenetre d'affichage du texte
        - texte : chaine de caractère à afficher
        - taille : taille de la chaine de caractère
        - p_x et p_y : coordonnées du point en haut à gauche
        - coul : couleur de la chaine de caractère à afficher
        - text_bold : affichage en gras. Argument optionnel (False par défaut)
        - text_italic : affichage en italique. Argument optionnel (False par défaut)
    """

    font = pygame.font.SysFont("vernada", taille, bold=text_bold, italic=text_italic)
    text = font.render(texte, 1, coul)
    fenetre.blit(text, (p_x, p_y))


#----------------------------------------------------------#
#-----------------                         ----------------#
#                    3. GESTION D'IMAGES                   #
#-----------------                         ----------------#
#----------------------------------------------------------#

def affiche_image(fenetre, nom_img, p_x, p_y):
    """
    Affiche une image.
    Paramètres d'entrées :
        - fenetre : fenetre d'affichage de l'image
        - nom_img : chaine de caractère donnant le nom du fichier image.
        - p_x et p_y : coordonnées du point en haut à gauche
    Renvoie l'image sous forme de surface pygame.
    """
    fond = pygame.image.load(nom_img).convert()
    fenetre.blit(fond, (p_x, p_y))
    return fond

def affiche_image_transp(fenetre, nom_img, p_x, p_y):
    """
    Affiche une image à transparence.
    Paramètres d'entrées :
        - fenetre : fenetre d'affichage de l'image
        - nom_img : chaine de caractère donnant le nom du fichier image.
        - p_x et p_y : coordonnées du point en haut à gauche
    Renvoie l'image sous forme de surface pygame.
    """
    fond = pygame.image.load(nom_img).convert_alpha()
    fenetre.blit(fond, (p_x, p_y))
    return fond

# A voir
def transfer_image(nom_img):
    """
    Renvoie la surface pygame correspondant à l'image F (fichier).
    Attention, l'image n'est pas affichée à l'écran ; cette fonction est
    utile pour obtenir les dimensions de l'image, la redimensionner ou la
    sauvegarder
    """
    return pygame.image.load(nom_img)

# A voir
def resize_image(I,W,H):
    """
    I est une surface pygame.
    W et H sont des entiers supérieurs à 1.
    Permet de redimensionner I avec la largeur W et la hauteur H.
    Renvoie la nouvelle surface pygame.
    """
    return pygame.transform.scale(I,(W,H))


#----------------------------------------------------------#
#-----------------                         ----------------#
#              4. GESTION DES SONS ET MUSIQUES             #
#-----------------                         ----------------#
#----------------------------------------------------------#

############################################################
# PARTIE 4.1 : SONS
############################################################

def play_sound(mon_son):
    """
    F est une chaîne de caractères contenant le nom du fichier son.
    Joue le son F.
    """
    pygame.mixer.Sound(mon_son).play()


def stop_sound(mon_son):
    """
    F est une chaîne de caractères contenant le nom du fichier son.
    Arrête le son F.
    """
    pygame.mixer.Sound(mon_son).stop()


def set_volume_sound(mon_son, vol):
    """
    F est une chaîne de caractères contenant le nom du fichier son.
    v est un flottant entre 0.0 et 1.0.
    Permet de régler le volume du son F.
    """
    pygame.mixer.Sound(mon_son).set_volume(vol)


def get_volume_sound(mon_son):
    """
    F est une chaîne de caractères.
    Renvoie le volume du son F.
    """
    return pygame.mixer.Sound(mon_son).get_volume()


############################################################
# PARTIE 4.2 : MUSIQUES
############################################################

def load_music(ma_music):
    """
    F est une chaîne de caractères contenant le nom du fichier audio.
    Charge la musique F mais ne la joue pas.
    Si une musique était déjà chargée, cela la stoppe si elle etait jouée.
    Utiliser de préférence des .wav
    """
    pygame.mixer.music.load(ma_music)


def play_music(loop=0):
    """
    Lance la musique. Si la musique était déjà jouée, elle reprend au début
    L'argument optionnel loop prend les valeurs 0 ou 1.
    Si loop vaut 1, alors la musique est jouée en boucle.
    """
    if loop == 1:
        pygame.mixer.music.play(loops = -1)
    else:
        pygame.mixer.music.play()


def restart_music():
    """
    Relance la musique à partir du début.
    """
    pygame.mixer.music.rewind()


def stop_music():
    """
    Arrête la musique.
    """
    pygame.mixer.music.stop()


def pause_music():
    """
    Met la musique en pause.
    """
    pygame.mixer.music.pause()


def unpause_music():
    """
    Reprend la musique
    """
    pygame.mixer.music.unpause()


def set_volume_music(vol):
    """
    v est un flottant entre 0.0 et 1.0.
    Permet de régler le volume de la musique.
    """
    pygame.mixer.music.set_volume(vol)


def get_volume_music() :
    """
    Renvoie le volume de la musique.
    """
    return pygame.mixer.music.get_volume()


def get_busy_music() :
    """
    Renvoie 1 si une musique est jouée et 0 sinon.
    """
    return pygame.mixer.music.get_busy()


#----------------------------------------------------------#
#-----------------                         ----------------#
#                    5 GESTION DU TEMPS                    #
#-----------------                         ----------------#
#----------------------------------------------------------#

def attendre(millisecondes) :
    """
    Attend le nombre de millisecondes passé en argument
    """
    if PYGAME_SDL_AFFICHAGE == 1 :
        affiche_all()

    pygame.time.delay(millisecondes)


def chrono_start() :
    """
    Démarre un chronomètre
    """
    global CHRONO
    CHRONO = pygame.time.get_ticks()


def chrono_val() :
    """
    Affiche le temps écoulé en millisecondes depuis le lancement du chronomètre
    """
    global CHRONO
    return pygame.time.get_ticks()-CHRONO


#----------------------------------------------------------#
#-----------------                         ----------------#
#                   5 VARIABLES COULEURS                   #
#-----------------                         ----------------#
#----------------------------------------------------------#

###########################################################
# variables couleur 18 principales
###########################################################
vert = (0, 255, 0)
rouge = (255, 0, 0)
bleu = (0, 0, 255)
cyan = (0, 255, 255)
blanc = (255, 255, 255)
jaune = (255, 255, 0)
orange = (255, 165, 0)
noir = (0, 0, 0)
argent = (192, 192, 192)
bleumarine = (0, 0, 128)
citronvert = (0, 255, 0)
magenta = (255, 0, 255)
gris = (128, 128, 128)
marron = (128, 0, 0)
sarcelle = (0, 128, 128)
vertclair = (0, 128, 0)
vertolive = (128, 128, 0)
violet = (128, 0, 128)

###########################################################
# variables couleur 140 en Anglais
###########################################################
aliceblue = (240, 248, 255)
antiquewhite = (250, 235, 215)
aqua = (0, 255, 255)
aquamarine = (127, 255, 212)
azure = (240, 255, 255)
beige = (245, 245, 220)
bisque = (255, 228, 196)
black = (0, 0, 0)
blanchedalmond = (255, 235, 205)
blue = (0, 0, 255)
blueviolet = (138, 43, 226)
brown = (165, 42, 42)
burlywood = (222, 184, 135)
cadetblue = (95, 158, 160)
chartreuse = (127, 255, 0)
chocolate = (210, 105, 30)
coral = (255, 127, 80)
cornflowerblue = (100, 149, 237)
cornsilk = (255, 248, 220)
crimson = (220, 20, 60)
darkblue = (0, 0, 139)
darkcyan = (0, 139, 139)
darkgoldenrod = (184, 134, 11)
darkgray = (169, 169, 169)
darkgreen = 0x006400
darkkhaki = 0xBDB76B
darkmagenta = 0x8B008B
darkolivegreen = 0x556B2F
darkorange = 0xFF8C00
darkorchid = 0x9932CC
darkred = 0x8B0000
darksalmon = 0xE9967A
darkseagreen = 0x8FBC8F
darkslateblue = 0x483D8B
darkslategray = 0x2F4F4F
darkturquoise = 0x00CED1
darkviolet = 0x9400D3
deeppink = 0xFF1493
deepskyblue = 0x00BFFF
dimgray = 0x696969
dodgerblue = 0x1E90FF
firebrick = 0xB22222
floralwhite = 0xFFFAF0
forestgreen = 0x228B22
fuchsia = 0xFF00FF
gainsboro = 0xDCDCDC
ghostwhite = 0xF8F8FF
gold = 0xFFD700
goldenrod = 0xDAA520
gray = 0x808080
green = 0x008000
greenyellow = 0xADFF2F
honeydew = 0xF0FFF0
hotpink = 0xFF69B4
indianred = 0xCD5C5C
indigo = 0x4B0082
ivory = 0xFFFFF0
khaki = 0xF0E68C
lavender = 0xE6E6FA
lavenderblush = 0xFFF0F5
lawngreen = 0x7CFC00
lemonchiffon = 0xFFFACD
lightblue = 0xADD8E6
lightcoral = 0xF08080
lightcyan = 0xE0FFFF
lightgoldenrodyellow = 0xFAFAD2
lightgreen = 0x90EE90
lightgrey = 0xD3D3D3
lightpink = 0xFFB6C1
lightsalmon = 0xFFA07A
lightseagreen = 0x20B2AA
lightskyblue = 0x87CEFA
lightslategray = 0x778899
lightsteelblue = 0xB0C4DE
lightyellow = 0xFFFFE0
lime = 0x00FF00
limegreen = 0x32CD32
linen = 0xFAF0E6
magenta = 0xFF00FF
maroon = 0x800000
mediumaquamarine = 0x66CDAA
mediumblue = 0x0000CD
mediumorchid = 0xBA55D3
mediumpurple = 0x9370DB
mediumseagreen = 0x3CB371
mediumslateblue = 0x7B68EE
mediumspringgreen = 0x00FA9A
mediumturquoise = 0x48D1CC
mediumvioletred = 0xC71585
midnightblue = 0x191970
mintcream = 0xF5FFFA
mistyrose = 0xFFE4E1
moccasin = 0xFFE4B5
navajowhite = 0xFFDEAD
navy = 0x000080
oldlace = 0xFDF5E6
olive = 0x808000
olivedrab = 0x6B8E23
orange = 0xFFA500
orangered = 0xFF4500
orchid = 0xDA70D6
palegoldenrod = 0xEEE8AA
palegreen = 0x98FB98
paleturquoise= 0xAFEEEE
palevioletred = 0xDB7093
papayawhip = 0xFFEFD5
peachpuff = 0xFFDAB9
peru = 0xCD853F
pink = 0xFFC0CB
plum = 0xDDA0DD
powderblue = 0xB0E0E6
purple = 0x800080
red = 0xFF0000
rosybrown = 0xBC8F8F
royalblue = 0x4169E1
saddlebrown = 0x8B4513
salmon = 0xFA8072
sandybrown = 0xF4A460
seagreen = 0x2E8B57
seashell = 0xFFF5EE
sienna = 0xA0522D
silver = 0xC0C0C0
skyblue = 0x87CEEB
slateblue = 0x6A5ACD
slategray = 0x708090
snow = 0xFFFAFA
springgreen = 0x00FF7F
steelblue = 0x4682B4
tan = 0xD2B48C
teal = 0x008080
thistle = 0xD8BFD8
tomato = 0xFF6347
turquoise = 0x40E0D0
violetlight = 0xEE82EE
wheat = 0xF5DEB3
white = 0xFFFFFF
whitesmoke = 0xF5F5F5
yellow = 0xFFFF00
yellowgreen = 0x9ACD32